@@ -31,8 +31,9 @@
 #include <gtk/gtkbox.h>
 #include <libgnomeui/gnome-help.h>
 #include <gdk/gdkkeysyms.h>
+#include <libgpm.h>
+
 #include "brightness-applet.h"
-#include "../../src/gpm-common.h"
 
 static void      gpm_brightness_applet_class_init (GpmBrightnessAppletClass *klass);
 static void      gpm_brightness_applet_init       (GpmBrightnessApplet *applet);
@@ -379,7 +380,7 @@ key_press_cb (GpmBrightnessApplet *applet, GdkEventKey *event)
 		return FALSE;
 		break;
 	}
-  
+
 	return FALSE;
 }
 
@@ -405,7 +406,7 @@ scroll_cb (GpmBrightnessApplet *applet, GdkEventScroll *event)
 	}
 	return FALSE;
 }
-	
+
 /**
  * create_popup:
  * @applet: Brightness applet instance
