@@ -45,7 +45,6 @@
 #include "gpm-conf.h"
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
-#include "gpm-hal.h"
 
 static void     gpm_dpms_class_init (GpmDpmsClass *klass);
 static void     gpm_dpms_init       (GpmDpms      *dpms);
@@ -53,14 +52,10 @@ static void     gpm_dpms_finalize   (GObject      *object);
 
 #define GPM_DPMS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_DPMS, GpmDpmsPrivate))
 
-/* until we get a nice event-emitting DPMS extension, we have to poll... */
-#define GPM_DPMS_POLL_TIME	10*1000
-
 struct GpmDpmsPrivate
 {
 	gboolean		 enabled;
 	gboolean		 active;
-	gboolean		 dpms_capable;
 
 	guint			 standby_timeout;
 	guint			 suspend_timeout;
@@ -68,7 +63,6 @@ struct GpmDpmsPrivate
 
 	GpmConf			*conf;
 	GpmDpmsMode		 mode;
-	GpmHal			*hal;
 
 	guint			 timer_id;
 };
@@ -85,7 +79,8 @@ enum {
 	PROP_OFF_TIMEOUT,
 };
 
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint signals [LAST_SIGNAL] = { 0, };
+static gpointer gpm_dpms_object = NULL;
 
 G_DEFINE_TYPE (GpmDpms, gpm_dpms, G_TYPE_OBJECT)
 
@@ -121,6 +116,9 @@ x11_sync_server_dpms_settings (Display *dpy,
 			       GError **error)
 {
 #ifdef HAVE_DPMS_EXTENSION
+
+	int	 event = 0;
+	int      err = 0;
 	BOOL	 o_enabled = FALSE;
 	CARD16	 o_power = 0;
 	CARD16	 o_standby = 0;
@@ -130,6 +128,25 @@ x11_sync_server_dpms_settings (Display *dpy,
 	gpm_debug ("Syncing DPMS settings enabled=%d timeouts=%d %d %d",
 		   enabled, standby_secs, suspend_secs, off_secs);
 
+	if (! DPMSQueryExtension (dpy, &event, &err)) {
+		gpm_debug ("XDPMS extension not supported");
+		g_set_error (error,
+			     GPM_DPMS_ERROR,
+			     GPM_DPMS_ERROR_GENERAL,
+			     "DPMS extension not supported");
+		return FALSE;
+	}
+
+	if (! DPMSCapable (dpy)) {
+		gpm_debug ("Display is not DPMS capable");
+		g_set_error (error,
+			     GPM_DPMS_ERROR,
+			     GPM_DPMS_ERROR_GENERAL,
+			     "Display is not DPMS capable");
+
+		return FALSE;
+	}
+
 	if (! DPMSInfo (dpy, &o_power, &o_enabled)) {
 		gpm_debug ("unable to get DPMS state.");
 		g_set_error (error,
@@ -205,12 +222,19 @@ x11_get_mode (GpmDpms     *dpms,
 	      GError     **error)
 {
 	GpmDpmsMode result;
-	BOOL enabled = FALSE;
-	CARD16 state;
+	int	    event_number;
+	int	    error_number;
+	BOOL	    enabled = FALSE;
+	CARD16	    state;
 
-	if (dpms->priv->dpms_capable == FALSE) {
-		/* Server or monitor can't DPMS -- assume the monitor is on. */
+	if (! DPMSQueryExtension (GDK_DISPLAY (), &event_number, &error_number)) {
+		/* Server doesn't know -- assume the monitor is on. */
 		result = GPM_DPMS_MODE_ON;
+
+	} else if (! DPMSCapable (GDK_DISPLAY ())) {
+		/* Server says the monitor doesn't do power management -- so it's on. */
+		result = GPM_DPMS_MODE_ON;
+
 	} else {
 		DPMSInfo (GDK_DISPLAY (), &state, &enabled);
 		if (! enabled) {
@@ -250,11 +274,22 @@ x11_set_mode (GpmDpms	 *dpms,
 	      GError    **error)
 {
 	GpmDpmsMode current_mode;
-	CARD16 state;
-	CARD16 current_state;
-	BOOL current_enabled;
+	CARD16	    state;
+	CARD16	    current_state;
+	BOOL	    current_enabled;
+	int	    event_number;
+	int	    error_number;
 
-	if (dpms->priv->dpms_capable == FALSE) {
+	if (! DPMSQueryExtension (GDK_DISPLAY (), &event_number, &error_number)) {
+		gpm_debug ("unable to query DPMS extention");
+		g_set_error (error,
+			     GPM_DPMS_ERROR,
+			     GPM_DPMS_ERROR_GENERAL,
+			     "DPMS extension not supported");
+		return FALSE;
+	}
+
+	if (! DPMSCapable (GDK_DISPLAY ())) {
 		gpm_debug ("not DPMS capable");
 		g_set_error (error,
 			     GPM_DPMS_ERROR,
@@ -358,16 +393,6 @@ sync_settings (GpmDpms *dpms,
 		off     = 0;
 	}
 
-	if (dpms->priv->dpms_capable == FALSE) {
-		gpm_debug ("Display is not DPMS capable");
-		g_set_error (error,
-			     GPM_DPMS_ERROR,
-			     GPM_DPMS_ERROR_GENERAL,
-			     "Display is not DPMS capable");
-
-		return FALSE;
-	}
-
 	/* We always try to keep the DPMS enabled so that
 	   we can use set the mode manually.  We will
 	   use zero values for the timeouts when the
@@ -483,12 +508,12 @@ gpm_dpms_method_from_string (const gchar *dpms_method)
 {
 	GpmDpmsMethod method;
 
-	/* default to unknown */	
+	/* default to unknown */
 	method = GPM_DPMS_METHOD_UNKNOWN;
 	if (dpms_method == NULL) {
 		return method;
 	}
-	
+
 	/* convert descriptive types to enumerated values */
 	if (strcmp (dpms_method, "default") == 0) {
 		method = GPM_DPMS_METHOD_DEFAULT;
@@ -787,10 +812,7 @@ gpm_dpms_init (GpmDpms *dpms)
 {
 	dpms->priv = GPM_DPMS_GET_PRIVATE (dpms);
 
-	/* DPMSCapable() can never change for a given display */
-	dpms->priv->dpms_capable = DPMSCapable (GDK_DISPLAY ());
-
-	add_poll_timer (dpms, GPM_DPMS_POLL_TIME);
+	add_poll_timer (dpms, 500);
 }
 
 static void
@@ -813,11 +835,11 @@ gpm_dpms_finalize (GObject *object)
 GpmDpms *
 gpm_dpms_new (void)
 {
-	static GpmDpms *dpms = NULL;
-	if (dpms != NULL) {
-		g_object_ref (dpms);
+	if (gpm_dpms_object != NULL) {
+		g_object_ref (gpm_dpms_object);
 	} else {
-		dpms = g_object_new (GPM_TYPE_DPMS, NULL);
+		gpm_dpms_object = g_object_new (GPM_TYPE_DPMS, NULL);
+		g_object_add_weak_pointer (gpm_dpms_object, &gpm_dpms_object);
 	}
-	return GPM_DPMS (dpms);
+	return GPM_DPMS (gpm_dpms_object);
 }
